/*
 * Copyright (C) 2012  Christian Mollekopf <mollekopf@kolabsys.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


#include "event.h"

#include <QDebug>
#include <QTimeZone>

#include <conversion/kcalconversion.h>
#include <conversion/commonconversion.h>
#include <kcalcore/memorycalendar.h>
#include <kcalcore/icalformat.h>

#include <iostream>
#include <kolabformat.h>
#include <kolabevent_p.h>

#include "libkolab-version.h"


namespace Kolab {
    namespace Calendaring {

Event::Event()
: Kolab::Event()
{
    setUid(Kolab::generateUID());
}

Event::Event(const Kolab::Event &e)
: Kolab::Event(e)
{
}

Event::~Event()
{

}


bool Event::read(const std::string &string)
{
    const Kolab::Event &e = Kolab::readEvent(string, false);
    if (Kolab::error()) {
        return false;
    }
    Kolab::Event::operator=(e);
    return true;
}

std::string Event::write() const
{
    return Kolab::writeEvent(*this);
}

bool contains(const Kolab::ContactReference &delegatorRef, const std::vector <Kolab::ContactReference > &list)
{
    foreach (const Kolab::ContactReference &ref, list) {
        if (delegatorRef.uid() == ref.uid() || delegatorRef.email() == ref.email() || delegatorRef.name() == ref.name()) {
            return true;
        }
    }
    return false;
}

void Event::delegate(const std::vector< Attendee >& delegators, const std::vector< Attendee >& delegatees)
{

    //First insert any missing attendees
    foreach(const Attendee &a, delegatees) {
        if (!getAttendee(a.contact())) {
            d->attendees.push_back(a);
        }
    }

    //Build a list of attendee references
    //These are pointers into d->attendees, so we MUST NOT modify that vector after this point!
    std::vector<Kolab::Attendee*> delegateesRef;
    foreach(const Attendee &a, delegatees) {
        Attendee *attendee = getAttendee(a.contact());
        Q_ASSERT(attendee);
        delegateesRef.push_back(attendee);
    }

    std::vector<Kolab::Attendee*> delegatorsRef;
    foreach(const Attendee& a, delegators) {
        if (Attendee *attendee = getAttendee(a.contact())) {
            delegatorsRef.push_back(attendee);
        } else {
            std::cout << "missing delegator";
        }
    }
    
    foreach (Attendee *delegatee, delegateesRef) {
        std::vector <Kolab::ContactReference > delegatedFrom = delegatee->delegatedFrom();
        foreach (Attendee *delegator, delegatorsRef) {

            //Set the delegator on each delegatee
            const ContactReference &delegatorRef = delegator->contact();
            if (!contains(delegatorRef, delegatedFrom)) {
                delegatedFrom.push_back(Kolab::ContactReference(Kolab::ContactReference::EmailReference, delegatorRef.email(), delegatorRef.name()));
            }

            //Set the delegatee on each delegator
            std::vector <Kolab::ContactReference > delegatedTo = delegator->delegatedTo();
            const ContactReference &delegaeeRef = delegatee->contact();
            if (!contains(delegaeeRef, delegatedTo)) {
                delegatedTo.push_back(Kolab::ContactReference(Kolab::ContactReference::EmailReference, delegaeeRef.email(), delegaeeRef.name()));
            }
            delegator->setDelegatedTo(delegatedTo);
        }
        delegatee->setDelegatedFrom(delegatedFrom);
    }
    
}

Attendee *Event::getAttendee(const ContactReference &ref)
{
    for(std::vector <Kolab::Attendee >::iterator it = d->attendees.begin();
        it != d->attendees.end(); it++) {
        if (it->contact().uid() == ref.uid() || it->contact().email() == ref.email() || it->contact().name() == ref.name()) {
            return &*it;
        }
    }
    return 0;
}


Attendee Event::getAttendee(const std::string &s)
{
    foreach(const Attendee &a, attendees()) {
        if (a.contact().uid() == s || a.contact().email() == s || a.contact().name() == s) {
            return a;
        }
    }
    return Attendee();
}

cDateTime Calendaring::Event::getNextOccurence(const cDateTime &date)
{
    KCalCore::Event::Ptr event = Kolab::Conversion::toKCalCore(*this);
    if (!event->recurs()) {
        return cDateTime();
    }
    const QDateTime nextDate = event->recurrence()->getNextDateTime(Kolab::Conversion::toDate(date));
    return Kolab::Conversion::fromDate(nextDate);
}


cDateTime Calendaring::Event::getOccurenceEndDate(const cDateTime &startDate)
{
    KCalCore::Event::Ptr event = Kolab::Conversion::toKCalCore(*this);
    const QDateTime start = Kolab::Conversion::toDate(startDate);
    return Kolab::Conversion::fromDate(event->endDateForStart(start));
}

cDateTime Calendaring::Event::getLastOccurrence() const
{
    KCalCore::Event::Ptr event = Kolab::Conversion::toKCalCore(*this);
    if (!event->recurs()) {
        return cDateTime();
    }
    const QDateTime endDate = event->recurrence()->endDateTime();
    return Kolab::Conversion::fromDate(endDate);
}

static std::vector< Kolab::Event > fromICalEvents(const std::string &input)
{
    KCalCore::Calendar::Ptr calendar(new KCalCore::MemoryCalendar(QTimeZone::utc()));
    KCalCore::ICalFormat format;
    format.setApplication("libkolab", LIBKOLAB_LIB_VERSION_STRING);
    format.fromString(calendar, Conversion::fromStdString(input));
    std::vector<Kolab::Event> events;
    QMap< QString, int > uidMap;
    foreach (const KCalCore::Event::Ptr &event, calendar->events()) {
        Kolab::Event e = Conversion::fromKCalCore(*event);
        if (!e.recurrenceID().isValid()) {
            if (uidMap.contains(event->uid())) {
                e.setExceptions(events.at(uidMap[event->uid()]).exceptions());
                events[uidMap[event->uid()]] = e;
            } else {
                events.push_back(e);
                uidMap.insert(event->uid(), events.size()-1);
            }
        } else {
            if (!uidMap.contains(event->uid())) {
                Kolab::Event e;
                e.setUid("");
                events.push_back(e);
                uidMap.insert(event->uid(), events.size()-1);
            }
            std::vector<Kolab::Event> exceptions = events.at( uidMap[event->uid()]).exceptions();
            exceptions.push_back(e);
            events.at(uidMap[event->uid()]).setExceptions(exceptions);
        }
    }
    return events;
}

static std::string eventsToICal(const std::vector<Kolab::Event> &events)
{
    KCalCore::Calendar::Ptr calendar(new KCalCore::MemoryCalendar(QTimeZone::utc()));
    foreach (const Kolab::Event &event, events) {
        KCalCore::Event::Ptr kcalEvent = Conversion::toKCalCore(event);
        if (!kcalEvent->created().isValid()) {
            kcalEvent->setCreated(QDateTime::currentDateTime()); //sets dtstamp
        }
        calendar->addEvent(kcalEvent);
        foreach(const Kolab::Event &exception, event.exceptions()) {
            KCalCore::Event::Ptr kcalException = Conversion::toKCalCore(exception);
            if (!kcalException->created().isValid()) {
                kcalException->setCreated(QDateTime::currentDateTime()); //sets dtstamp
            }
            calendar->addEvent(kcalException);
        }
    }
    KCalCore::ICalFormat format;
    format.setApplication("libkolab", LIBKOLAB_LIB_VERSION_STRING);
//     qDebug() << format.createScheduleMessage(calendar->events().first(), KCalCore::iTIPRequest);
    return Conversion::toStdString(format.toString(calendar));

}

std::string Event::toICal() const
{
    std::vector<Kolab::Event> list;
    list.push_back(*this);
    return eventsToICal(list);
}

bool Event::fromICal(const std::string &input)
{
    std::vector<Kolab::Event> list = fromICalEvents(input);
    if (list.size() != 1) {
        std::cout << "invalid number of events: " << list.size();
        return false;
    }
    Kolab::Event::operator=(list.at(0));
    return true;
}

    };
};
